@@ -2,7 +2,6 @@ package config
 
 import (
 	"github.com/deviceinsight/kafkactl/v5/internal/global"
-	"github.com/deviceinsight/kafkactl/v5/internal/output"
 	"github.com/pkg/errors"
 
 	"strings"
@@ -19,7 +18,7 @@ func newUseContextCmd() *cobra.Command {
 		Short:   "switch active context",
 		Long:    `command to switch active context`,
 		Args:    cobra.MinimumNArgs(1),
-		Run: func(_ *cobra.Command, args []string) {
+		RunE: func(_ *cobra.Command, args []string) error {
 
 			context := strings.Join(args, " ")
 
@@ -27,12 +26,13 @@ func newUseContextCmd() *cobra.Command {
 
 			// check if it is an existing context
 			if _, ok := contexts[context]; !ok {
-				output.Fail(errors.Errorf("not a valid context: %s", context))
+				return errors.Errorf("not a valid context: %s", context)
 			}
 
 			if err := global.SetCurrentContext(context); err != nil {
-				output.Fail(errors.Wrap(err, "unable to write config"))
+				return errors.Wrap(err, "unable to write config")
 			}
+			return nil
 		},
 		ValidArgsFunction: func(_ *cobra.Command, args []string, _ string) ([]string, cobra.ShellCompDirective) {
 			if len(args) != 0 {
