@@ -8,7 +8,9 @@ import (
 	"testing"
 	"time"
 
+	"github.com/deviceinsight/kafkactl/v5/internal"
 	"github.com/deviceinsight/kafkactl/v5/internal/helpers/protobuf"
+	"github.com/riferrei/srclient"
 
 	"github.com/deviceinsight/kafkactl/v5/internal/testutil"
 	"github.com/jhump/protoreflect/dynamic"
@@ -16,7 +18,6 @@ import (
 )
 
 func TestConsumeWithKeyAndValueIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 
 	topicName := testutil.CreateTopic(t, "consume-topic")
@@ -33,7 +34,6 @@ func TestConsumeWithKeyAndValueIntegration(t *testing.T) {
 }
 
 func TestConsumeWithPartitionAndValueIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 
 	topicName := testutil.CreateTopic(t, "consume-topic", "--partitions", "2")
@@ -50,7 +50,6 @@ func TestConsumeWithPartitionAndValueIntegration(t *testing.T) {
 }
 
 func TestConsumeWithEmptyPartitionsIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 
 	topicName := testutil.CreateTopic(t, "consume-topic", "--partitions", "10")
@@ -67,7 +66,6 @@ func TestConsumeWithEmptyPartitionsIntegration(t *testing.T) {
 }
 
 func TestConsumeTailIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 
 	topicName := testutil.CreateTopic(t, "consume-topic", "--partitions", "10")
@@ -94,8 +92,7 @@ func TestConsumeTailIntegration(t *testing.T) {
 	testutil.AssertEquals(t, "test-key-2#test-value-2b", messages[2])
 }
 
-func TestConsumeFromTimestamp(t *testing.T) {
-
+func TestConsumeFromTimestampIntegration(t *testing.T) {
 	testutil.StartIntegrationTest(t)
 
 	topicName := testutil.CreateTopic(t, "consume-topic", "--partitions", "2")
@@ -117,25 +114,25 @@ func TestConsumeFromTimestamp(t *testing.T) {
 	testutil.ProduceMessageOnPartition(t, topicName, "key-2", "g", 1, 3)
 	testutil.ProduceMessageOnPartition(t, topicName, "key-1", "h", 0, 3)
 
-	//test --from-timestamp with --to-timestamp with formatted dates
+	// test --from-timestamp with --to-timestamp with formatted dates
 	kafkaCtl := testutil.CreateKafkaCtlCommand()
-	t1Formatted := time.UnixMilli(t1).Format("2006-01-02T15:04:05.123Z")
-	t2Formatted := time.UnixMilli(t2).Format("2006-01-02T15:04:05.123Z")
+	t1Formatted := time.UnixMilli(t1).Format("2006-01-02T15:04:05.000Z")
+	t2Formatted := time.UnixMilli(t2).Format("2006-01-02T15:04:05.000Z")
 	if _, err := kafkaCtl.Execute("consume", topicName, "--from-timestamp", t1Formatted, "--to-timestamp", t2Formatted); err != nil {
 		t.Fatalf("failed to execute command: %v", err)
 	}
 	messages := strings.Split(strings.TrimSpace(kafkaCtl.GetStdOut()), "\n")
 	testutil.AssertArraysEquals(t, []string{"c", "d", "e", "f"}, messages)
 
-	//test --from-timestamp with --to-timestamp with unix epoch millis timestamps
+	// test --from-timestamp with --to-timestamp with unix epoch millis timestamps
 	kafkaCtl = testutil.CreateKafkaCtlCommand()
-	if _, err := kafkaCtl.Execute("consume", topicName, "--from-timestamp", strconv.FormatInt(t2, 10), "--to-timestamp", strconv.FormatInt(t2, 10)); err != nil {
+	if _, err := kafkaCtl.Execute("consume", topicName, "--from-timestamp", strconv.FormatInt(t1, 10), "--to-timestamp", strconv.FormatInt(t2, 10)); err != nil {
 		t.Fatalf("failed to execute command: %v", err)
 	}
 	messages = strings.Split(strings.TrimSpace(kafkaCtl.GetStdOut()), "\n")
 	testutil.AssertArraysEquals(t, []string{"c", "d", "e", "f"}, messages)
 
-	//test --from-timestamp with --max-messages (--partitions present for reproducibility)
+	// test --from-timestamp with --max-messages (--partitions present for reproducibility)
 	kafkaCtl = testutil.CreateKafkaCtlCommand()
 	if _, err := kafkaCtl.Execute("consume", topicName, "--from-timestamp", strconv.FormatInt(t1, 10), "--max-messages", strconv.Itoa(2), "--partitions", strconv.Itoa(1)); err != nil {
 		t.Fatalf("failed to execute command: %v", err)
@@ -143,7 +140,7 @@ func TestConsumeFromTimestamp(t *testing.T) {
 	messages = strings.Split(strings.TrimSpace(kafkaCtl.GetStdOut()), "\n")
 	testutil.AssertArraysEquals(t, []string{"c", "d"}, messages)
 
-	//test --from-timestamp with --exit
+	// test --from-timestamp with --exit
 	kafkaCtl = testutil.CreateKafkaCtlCommand()
 	if _, err := kafkaCtl.Execute("consume", topicName, "--from-timestamp", strconv.FormatInt(t2, 10), "--exit"); err != nil {
 		t.Fatalf("failed to execute command: %v", err)
@@ -152,8 +149,7 @@ func TestConsumeFromTimestamp(t *testing.T) {
 	testutil.AssertArraysEquals(t, []string{"g", "h"}, messages)
 }
 
-func TestConsumeToTimestamp(t *testing.T) {
-
+func TestConsumeToTimestampIntegration(t *testing.T) {
 	testutil.StartIntegrationTest(t)
 
 	topicName := testutil.CreateTopic(t, "consume-topic", "--partitions", "2")
@@ -175,7 +171,7 @@ func TestConsumeToTimestamp(t *testing.T) {
 	testutil.ProduceMessageOnPartition(t, topicName, "key-2", "g", 1, 3)
 	testutil.ProduceMessageOnPartition(t, topicName, "key-1", "h", 0, 3)
 
-	//test --from-beginning with --to-timestamp
+	// test --from-beginning with --to-timestamp
 	kafkaCtl := testutil.CreateKafkaCtlCommand()
 	if _, err := kafkaCtl.Execute("consume", topicName, "--from-beginning", "--to-timestamp", strconv.FormatInt(t1, 10)); err != nil {
 		t.Fatalf("failed to execute command: %v", err)
@@ -183,7 +179,7 @@ func TestConsumeToTimestamp(t *testing.T) {
 	messages := strings.Split(strings.TrimSpace(kafkaCtl.GetStdOut()), "\n")
 	testutil.AssertArraysEquals(t, []string{"a", "b"}, messages)
 
-	//test --to-timestamp with --tail
+	// test --to-timestamp with --tail
 	kafkaCtl = testutil.CreateKafkaCtlCommand()
 	if _, err := kafkaCtl.Execute("consume", topicName, "--to-timestamp", strconv.FormatInt(t2, 10), "--tail", strconv.Itoa(4)); err != nil {
 		t.Fatalf("failed to execute command: %v", err)
@@ -193,7 +189,6 @@ func TestConsumeToTimestamp(t *testing.T) {
 }
 
 func TestConsumeWithKeyAndValueAsBase64Integration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 
 	topicName := testutil.CreateTopic(t, "consume-topic")
@@ -213,7 +208,6 @@ func TestConsumeWithKeyAndValueAsBase64Integration(t *testing.T) {
 }
 
 func TestConsumeWithKeyAndValueAsHexIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 
 	topicName := testutil.CreateTopic(t, "consume-topic")
@@ -233,7 +227,6 @@ func TestConsumeWithKeyAndValueAsHexIntegration(t *testing.T) {
 }
 
 func TestConsumeWithKeyAndValueAutoDetectBinaryValueIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 
 	topicName := testutil.CreateTopic(t, "consume-topic")
@@ -259,7 +252,6 @@ func TestConsumeWithKeyAndValueAutoDetectBinaryValueIntegration(t *testing.T) {
 }
 
 func TestAvroDeserializationErrorHandlingIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 
 	valueSchema := `{
@@ -275,7 +267,7 @@ func TestAvroDeserializationErrorHandlingIntegration(t *testing.T) {
 	value := `{"name":"Peter Mueller"}`
 	value2 := `{"name":"Peter Pan"}`
 
-	topicName := testutil.CreateAvroTopic(t, "avro-topic", "", valueSchema)
+	topicName := testutil.CreateTopicWithSchema(t, "avro-topic", "", valueSchema, srclient.Avro)
 
 	group := testutil.CreateConsumerGroup(t, "avro-topic-consumer-group", topicName)
 
@@ -307,19 +299,23 @@ func TestAvroDeserializationErrorHandlingIntegration(t *testing.T) {
 	testutil.AssertEquals(t, "message produced (partition=0\toffset=2)", kafkaCtl.GetStdOut())
 
 	if _, err := kafkaCtl.Execute("consume", topicName, "--from-beginning", "--exit"); err != nil {
-		testutil.AssertErrorContains(t, "failed to find avro schema", err)
-		testutil.AssertEquals(t, value, kafkaCtl.GetStdOut())
+		t.Fatalf("failed to execute command: %v", err)
 	} else {
-		t.Fatalf("expected consumer to fail")
+		results := strings.Split(strings.TrimSpace(kafkaCtl.GetStdOut()), "\n")
+		testutil.AssertContains(t, value, results)
+		testutil.AssertContains(t, "no-avro", results)
+		testutil.AssertContains(t, value2, results)
 	}
 
 	kafkaCtl = testutil.CreateKafkaCtlCommand()
 
 	if _, err := kafkaCtl.Execute("consume", topicName, "--group", group, "--max-messages", "3"); err != nil {
-		testutil.AssertErrorContains(t, "failed to find avro schema", err)
-		testutil.AssertEquals(t, value, kafkaCtl.GetStdOut())
+		t.Fatalf("failed to execute command: %v", err)
 	} else {
-		t.Fatalf("expected consumer to fail")
+		results := strings.Split(strings.TrimSpace(kafkaCtl.GetStdOut()), "\n")
+		testutil.AssertContains(t, value, results)
+		testutil.AssertContains(t, "no-avro", results)
+		testutil.AssertContains(t, value2, results)
 	}
 }
 
@@ -332,7 +328,7 @@ func TestProtobufConsumeProtoFileIntegration(t *testing.T) {
 
 	protoPath := filepath.Join(testutil.RootDir, "internal", "testutil", "testdata")
 	now := time.Date(2021, time.December, 1, 14, 10, 12, 0, time.UTC)
-	pbMessageDesc := protobuf.ResolveMessageType(protobuf.SearchContext{
+	pbMessageDesc := protobuf.ResolveMessageType(internal.ProtobufConfig{
 		ProtoImportPaths: []string{protoPath},
 		ProtoFiles:       []string{"msg.proto"},
 	}, "TopicMessage")
@@ -368,7 +364,7 @@ func TestProtobufConsumeProtoFileWithoutProtoImportPathIntegration(t *testing.T)
 
 	protoPath := filepath.Join(testutil.RootDir, "internal", "testutil", "testdata")
 	now := time.Date(2021, time.December, 1, 14, 10, 12, 0, time.UTC)
-	pbMessageDesc := protobuf.ResolveMessageType(protobuf.SearchContext{
+	pbMessageDesc := protobuf.ResolveMessageType(internal.ProtobufConfig{
 		ProtoImportPaths: []string{protoPath},
 		ProtoFiles:       []string{"msg.proto"},
 	}, "TopicMessage")
@@ -424,7 +420,7 @@ func TestProtobufConsumeProtosetFileIntegration(t *testing.T) {
 
 	protoPath := filepath.Join(testutil.RootDir, "internal", "testutil", "testdata", "msg.protoset")
 	now := time.Date(2021, time.December, 1, 14, 10, 12, 0, time.UTC)
-	pbMessageDesc := protobuf.ResolveMessageType(protobuf.SearchContext{
+	pbMessageDesc := protobuf.ResolveMessageType(internal.ProtobufConfig{
 		ProtosetFiles: []string{protoPath},
 	}, "TopicMessage")
 	pbMessage := dynamic.NewMessage(pbMessageDesc)
@@ -458,6 +454,25 @@ func TestProtobufConsumeProtoFileErrNoMessageIntegration(t *testing.T) {
 	kafkaCtl := testutil.CreateKafkaCtlCommand()
 
 	protoPath := filepath.Join(testutil.RootDir, "internal", "testutil", "testdata", "msg.protoset")
+	now := time.Date(2021, time.December, 1, 14, 10, 12, 0, time.UTC)
+	pbMessageDesc := protobuf.ResolveMessageType(internal.ProtobufConfig{
+		ProtosetFiles: []string{protoPath},
+	}, "TopicMessage")
+	pbMessage := dynamic.NewMessage(pbMessageDesc)
+	pbMessage.SetFieldByNumber(1, timestamppb.New(now))
+	pbMessage.SetFieldByNumber(2, int64(1))
+
+	value, err := pbMessage.Marshal()
+	if err != nil {
+		t.Fatalf("Failed to marshal proto message: %s", err)
+	}
+
+	// produce valid pb message
+	if _, err := kafkaCtl.Execute("produce", pbTopic, "--key", "test-key", "--value", hex.EncodeToString(value), "--value-encoding", "hex"); err != nil {
+		t.Fatalf("failed to execute command: %v", err)
+	}
+
+	testutil.AssertEquals(t, "message produced (partition=0\toffset=0)", kafkaCtl.GetStdOut())
 
 	if _, err := kafkaCtl.Execute("consume", pbTopic, "--from-beginning", "--exit", "--proto-import-path", protoPath, "--proto-file", "msg.proto", "--value-proto-type", "NonExisting"); err != nil {
 		testutil.AssertErrorContains(t, "not found in provided files", err)
@@ -483,14 +498,13 @@ func TestProtobufConsumeProtoFileErrDecodeIntegration(t *testing.T) {
 	testutil.AssertEquals(t, "message produced (partition=0\toffset=0)", kafkaCtl.GetStdOut())
 
 	if _, err := kafkaCtl.Execute("consume", pbTopic, "--from-beginning", "--exit", "--proto-import-path", protoPath, "--proto-file", "msg.proto", "--value-proto-type", "TopicMessage"); err != nil {
-		testutil.AssertErrorContains(t, "value decode failed: proto: bad wiretype", err)
+		testutil.AssertErrorContains(t, "failed to deserialize value: proto: bad wiretype", err)
 	} else {
 		t.Fatal("Expected consumer to fail")
 	}
 }
 
 func TestConsumeGroupIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 
 	prefix := "consume-group-"
@@ -524,7 +538,6 @@ func TestConsumeGroupIntegration(t *testing.T) {
 }
 
 func TestConsumeAutoCompletionIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 
 	prefix := "consume-complete-"
@@ -548,7 +561,6 @@ func TestConsumeAutoCompletionIntegration(t *testing.T) {
 }
 
 func TestConsumeGroupCompletionIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 
 	prefix := "consume-group-complete-"
@@ -574,7 +586,6 @@ func TestConsumeGroupCompletionIntegration(t *testing.T) {
 }
 
 func TestConsumePartitionsK8sIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTestWithContext(t, "k8s-mock")
 
 	kafkaCtl := testutil.CreateKafkaCtlCommand()
@@ -603,7 +614,6 @@ func TestConsumePartitionsK8sIntegration(t *testing.T) {
 		},
 	} {
 		t.Run(test.description, func(t *testing.T) {
-
 			if _, err := kafkaCtl.Execute(test.args...); err != nil {
 				t.Fatalf("failed to execute command: %v", err)
 			}
