@@ -10,7 +10,9 @@ import (
 	"strings"
 	"testing"
 
+	"github.com/deviceinsight/kafkactl/v5/internal"
 	"github.com/deviceinsight/kafkactl/v5/internal/helpers/protobuf"
+	"github.com/riferrei/srclient"
 
 	"github.com/jhump/protoreflect/dynamic"
 
@@ -18,7 +20,6 @@ import (
 )
 
 func TestProduceWithKeyAndValueIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 
 	topicName := testutil.CreateTopic(t, "produce-topic")
@@ -39,7 +40,6 @@ func TestProduceWithKeyAndValueIntegration(t *testing.T) {
 }
 
 func TestProduceMessageWithHeadersIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 
 	topicName := testutil.CreateTopic(t, "produce-topic")
@@ -60,7 +60,6 @@ func TestProduceMessageWithHeadersIntegration(t *testing.T) {
 }
 
 func TestProduceAvroMessageWithHeadersIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 
 	valueSchema := `{
@@ -75,7 +74,7 @@ func TestProduceAvroMessageWithHeadersIntegration(t *testing.T) {
 }`
 	value := `{"name":"Peter Mueller"}`
 
-	topicName := testutil.CreateAvroTopic(t, "produce-topic", "", valueSchema)
+	topicName := testutil.CreateTopicWithSchema(t, "produce-topic", "", valueSchema, srclient.Avro)
 
 	kafkaCtl := testutil.CreateKafkaCtlCommand()
 
@@ -92,8 +91,7 @@ func TestProduceAvroMessageWithHeadersIntegration(t *testing.T) {
 	testutil.AssertEquals(t, fmt.Sprintf("key1:value1,key\\:2:value\\:2#test-key#%s", value), kafkaCtl.GetStdOut())
 }
 
-func TestProduceAvroMessageOmitDefaultValue(t *testing.T) {
-
+func TestProduceAvroMessageOmitDefaultValueIntegration(t *testing.T) {
 	testutil.StartIntegrationTest(t)
 
 	valueSchema := `{
@@ -109,7 +107,7 @@ func TestProduceAvroMessageOmitDefaultValue(t *testing.T) {
 	 "CurrencyCode": "EUR"
 	}`
 
-	topicName := testutil.CreateAvroTopic(t, "produce-avro-topic", "", valueSchema)
+	topicName := testutil.CreateTopicWithSchema(t, "produce-avro-topic", "", valueSchema, srclient.Avro)
 
 	kafkaCtl := testutil.CreateKafkaCtlCommand()
 
@@ -128,8 +126,7 @@ func TestProduceAvroMessageOmitDefaultValue(t *testing.T) {
 	testutil.AssertContainSubstring(t, `"ExpiresOn":null`, stdout)
 }
 
-func TestProduceAvroMessageWithUnionStandardJson(t *testing.T) {
-
+func TestProduceAvroMessageWithUnionStandardJsonIntegration(t *testing.T) {
 	testutil.StartIntegrationTest(t)
 
 	valueSchema := `{
@@ -147,7 +144,7 @@ func TestProduceAvroMessageWithUnionStandardJson(t *testing.T) {
 	 "ExpiresOn": "2022-12-12"
 	}`
 
-	topicName := testutil.CreateAvroTopic(t, "produce-topic", "", valueSchema)
+	topicName := testutil.CreateTopicWithSchema(t, "produce-topic", "", valueSchema, srclient.Avro)
 
 	kafkaCtl := testutil.CreateKafkaCtlCommand()
 
@@ -166,8 +163,111 @@ func TestProduceAvroMessageWithUnionStandardJson(t *testing.T) {
 	testutil.AssertContainSubstring(t, `"ExpiresOn":"2022-12-12"`, stdout)
 }
 
-func TestProduceAvroMessageWithUnionAvroJson(t *testing.T) {
+func TestProduceRegistryProtobufMessageWithHeadersIntegration(t *testing.T) {
+	testutil.StartIntegrationTest(t)
 
+	valueSchema := `syntax = "proto3";
+  package foo.bar;
+
+  message Msg {
+    string name = 1;
+  }`
+	value := `{"name":"Peter Mueller"}`
+
+	topicName := testutil.CreateTopicWithSchema(t, "produce-protobuf-topic", "", valueSchema, srclient.Protobuf)
+
+	kafkaCtl := testutil.CreateKafkaCtlCommand()
+
+	if _, err := kafkaCtl.Execute("produce", topicName, "--key", "test-key", "--value", value, "-H", "key1:value1", "-H", "key\\:2:value\\:2"); err != nil {
+		t.Fatalf("failed to execute command: %v", err)
+	}
+
+	testutil.AssertEquals(t, "message produced (partition=0\toffset=0)", kafkaCtl.GetStdOut())
+
+	if _, err := kafkaCtl.Execute("consume", topicName, "--from-beginning", "--exit", "--print-keys", "--print-headers"); err != nil {
+		t.Fatalf("failed to execute command: %v", err)
+	}
+
+	testutil.AssertEquals(t, fmt.Sprintf("key1:value1,key\\:2:value\\:2#test-key#%s", value), kafkaCtl.GetStdOut())
+}
+
+func TestProduceRegistryProtobufMessageOmitDefaultValueIntegration(t *testing.T) {
+	testutil.StartIntegrationTest(t)
+
+	valueSchema := `syntax = "proto3";
+  package foo.bar;
+
+  message Msg {
+    string current_code = 1;
+    string expires_on = 2;
+  }`
+	value := `{"currentCode":"EUR"}`
+
+	topicName := testutil.CreateTopicWithSchema(t, "produce-protobuf-topic", "", valueSchema, srclient.Protobuf)
+
+	kafkaCtl := testutil.CreateKafkaCtlCommand()
+
+	if _, err := kafkaCtl.Execute("produce", topicName, "--value", value); err != nil {
+		t.Fatalf("failed to execute command: %v", err)
+	}
+
+	testutil.AssertEquals(t, "message produced (partition=0\toffset=0)", kafkaCtl.GetStdOut())
+
+	if _, err := kafkaCtl.Execute("consume", topicName, "--from-beginning", "--exit"); err != nil {
+		t.Fatalf("failed to execute command: %v", err)
+	}
+
+	stdout := kafkaCtl.GetStdOut()
+	testutil.AssertContainSubstring(t, `"currentCode":"EUR"`, stdout)
+	testutil.AssertContainSubstring(t, `"expiresOn":""`, stdout)
+}
+
+func TestProduceJsonMessageWithSchemaIntegration(t *testing.T) {
+	testutil.StartIntegrationTest(t)
+
+	valueSchema := `{
+	  "$schema": "http://json-schema.org/draft-04/schema#",
+	  "type": "object",
+	  "properties": {
+		"CurrencyCode": {
+		  "type": "string"
+		},
+		"ExpiresOn": {
+		  "type": "string",
+		  "format": "date"
+		}
+	  },
+	  "required": [
+		"CurrencyCode",
+		"ExpiresOn"
+	  ]
+	}`
+
+	value := `{
+	 "CurrencyCode": "EUR",
+	 "ExpiresOn": "2022-12-12"
+	}`
+
+	topicName := testutil.CreateTopicWithSchema(t, "produce-topic", "", valueSchema, srclient.Json)
+
+	kafkaCtl := testutil.CreateKafkaCtlCommand()
+
+	if _, err := kafkaCtl.Execute("produce", topicName, "--value", value); err != nil {
+		t.Fatalf("failed to execute command: %v", err)
+	}
+
+	testutil.AssertEquals(t, "message produced (partition=0\toffset=0)", kafkaCtl.GetStdOut())
+
+	if _, err := kafkaCtl.Execute("consume", topicName, "--from-beginning", "--exit"); err != nil {
+		t.Fatalf("failed to execute command: %v", err)
+	}
+
+	stdout := kafkaCtl.GetStdOut()
+	testutil.AssertContainSubstring(t, `"CurrencyCode": "EUR"`, stdout)
+	testutil.AssertContainSubstring(t, `"ExpiresOn": "2022-12-12"`, stdout)
+}
+
+func TestProduceAvroMessageWithUnionAvroJsonIntegration(t *testing.T) {
 	testutil.StartIntegrationTest(t)
 
 	valueSchema := `{
@@ -189,7 +289,7 @@ func TestProduceAvroMessageWithUnionAvroJson(t *testing.T) {
 		t.Fatalf("unable to set env variable: %v", err)
 	}
 
-	topicName := testutil.CreateAvroTopic(t, "produce-topic", "", valueSchema)
+	topicName := testutil.CreateTopicWithSchema(t, "produce-topic", "", valueSchema, srclient.Avro)
 
 	kafkaCtl := testutil.CreateKafkaCtlCommand()
 
@@ -209,7 +309,6 @@ func TestProduceAvroMessageWithUnionAvroJson(t *testing.T) {
 }
 
 func TestProduceTombstoneIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 
 	topicName := testutil.CreateTopic(t, "produce-topic")
@@ -231,7 +330,6 @@ func TestProduceTombstoneIntegration(t *testing.T) {
 }
 
 func TestProduceFromBase64Integration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 
 	topicName := testutil.CreateTopic(t, "produce-topic")
@@ -254,7 +352,6 @@ func TestProduceFromBase64Integration(t *testing.T) {
 }
 
 func TestProduceFromHexIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 
 	topicName := testutil.CreateTopic(t, "produce-topic")
@@ -277,7 +374,6 @@ func TestProduceFromHexIntegration(t *testing.T) {
 }
 
 func TestProduceAutoCompletionIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 
 	prefix := "produce-complete-"
@@ -337,11 +433,11 @@ func TestProduceProtoFileIntegration(t *testing.T) {
 		t.Fatalf("Failed to decode value: %s", err)
 	}
 
-	keyMessage := dynamic.NewMessage(protobuf.ResolveMessageType(protobuf.SearchContext{
+	keyMessage := dynamic.NewMessage(protobuf.ResolveMessageType(internal.ProtobufConfig{
 		ProtoImportPaths: []string{protoPath},
 		ProtoFiles:       []string{"msg.proto"},
 	}, "TopicKey"))
-	valueMessage := dynamic.NewMessage(protobuf.ResolveMessageType(protobuf.SearchContext{
+	valueMessage := dynamic.NewMessage(protobuf.ResolveMessageType(internal.ProtobufConfig{
 		ProtoImportPaths: []string{protoPath},
 		ProtoFiles:       []string{"msg.proto"},
 	}, "TopicMessage"))
@@ -368,7 +464,6 @@ func TestProduceProtoFileIntegration(t *testing.T) {
 }
 
 func TestProduceWithCSVFileIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 	topic := testutil.CreateTopic(t, "produce-topic-csv")
 	kafkaCtl := testutil.CreateKafkaCtlCommand()
@@ -390,7 +485,6 @@ func TestProduceWithCSVFileIntegration(t *testing.T) {
 }
 
 func TestProduceWithCSVFileWithTimestampsFirstColumnIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 	topic := testutil.CreateTopic(t, "produce-topic-csv")
 	kafkaCtl := testutil.CreateKafkaCtlCommand()
@@ -412,7 +506,6 @@ func TestProduceWithCSVFileWithTimestampsFirstColumnIntegration(t *testing.T) {
 }
 
 func TestProduceWithCSVFileWithTimestampsSecondColumnIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 	topic := testutil.CreateTopic(t, "produce-topic-csv")
 	kafkaCtl := testutil.CreateKafkaCtlCommand()
@@ -434,7 +527,6 @@ func TestProduceWithCSVFileWithTimestampsSecondColumnIntegration(t *testing.T) {
 }
 
 func TestProduceWithJSONFileIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 	topic := testutil.CreateTopic(t, "produce-topic-json")
 	kafkaCtl := testutil.CreateKafkaCtlCommand()
@@ -457,7 +549,6 @@ func TestProduceWithJSONFileIntegration(t *testing.T) {
 }
 
 func TestProduceWithJSONFileBase64ValuesIntegration(t *testing.T) {
-
 	testutil.StartIntegrationTest(t)
 	topic := testutil.CreateTopic(t, "produce-topic-json-base64-values")
 	kafkaCtl := testutil.CreateKafkaCtlCommand()
@@ -511,7 +602,7 @@ func TestProduceProtoFileWithOnlyKeyEncodedIntegration(t *testing.T) {
 		t.Fatalf("Failed to decode key: %s", err)
 	}
 
-	keyMessage := dynamic.NewMessage(protobuf.ResolveMessageType(protobuf.SearchContext{
+	keyMessage := dynamic.NewMessage(protobuf.ResolveMessageType(internal.ProtobufConfig{
 		ProtoImportPaths: []string{protoPath},
 		ProtoFiles:       []string{"msg.proto"},
 	}, "TopicKey"))
@@ -568,11 +659,11 @@ func TestProduceProtoFileWithoutProtoImportPathIntegration(t *testing.T) {
 		t.Fatalf("Failed to decode value: %s", err)
 	}
 
-	keyMessage := dynamic.NewMessage(protobuf.ResolveMessageType(protobuf.SearchContext{
+	keyMessage := dynamic.NewMessage(protobuf.ResolveMessageType(internal.ProtobufConfig{
 		ProtoImportPaths: []string{protoPath},
 		ProtoFiles:       []string{"msg.proto"},
 	}, "TopicKey"))
-	valueMessage := dynamic.NewMessage(protobuf.ResolveMessageType(protobuf.SearchContext{
+	valueMessage := dynamic.NewMessage(protobuf.ResolveMessageType(internal.ProtobufConfig{
 		ProtoImportPaths: []string{protoPath},
 		ProtoFiles:       []string{"msg.proto"},
 	}, "TopicMessage"))
@@ -635,10 +726,10 @@ func TestProduceProtosetFileIntegration(t *testing.T) {
 		t.Fatalf("Failed to decode value: %s", err)
 	}
 
-	keyMessage := dynamic.NewMessage(protobuf.ResolveMessageType(protobuf.SearchContext{
+	keyMessage := dynamic.NewMessage(protobuf.ResolveMessageType(internal.ProtobufConfig{
 		ProtosetFiles: []string{protoPath},
 	}, "TopicKey"))
-	valueMessage := dynamic.NewMessage(protobuf.ResolveMessageType(protobuf.SearchContext{
+	valueMessage := dynamic.NewMessage(protobuf.ResolveMessageType(internal.ProtobufConfig{
 		ProtosetFiles: []string{protoPath},
 	}, "TopicMessage"))
 
